# 
# Metadata mode
#
# ver 2.00+
# Brian Carrier [carrier@sleuthkit.org]
# Copyright (c) 2003-2004 by Brian Carrier.  All rights reserved
#
# This file is part of the Autopsy Forensic Browser (Autopsy)
#
# Autopsy is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Autopsy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Autopsy; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR PURPOSE.
# IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA, OR PROFITS OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


package Meta;

$Meta::FRAME = 0;
$Meta::ENTER = 1;
$Meta::STATS = 2;
$Meta::EXPORT = 3;
$Meta::FFIND = 4;
$Meta::LIST = 5;
$Meta::REPORT = 6;
$Meta::BLANK = 7;


sub main {

    # By default, show the main frame
    $Args::args{'view'} = $Args::enc_args{'view'} = $Meta::FRAME    
          unless (exists $Args::args{'view'});

    Args::check_view();    
    my $view = Args::get_view();

    if ($view == $Meta::BLANK) {
        blank();	
        return 0;
    }


    # Check Basic Args
    Args::check_img('img');
    Args::check_ftype();
    Args::check_mnt();


    # These windows don't need the meta data address
    if ($view == $Meta::FRAME) {
        return frame();	
    }
    elsif ($view == $Meta::ENTER) {
        return enter();	
    }
    elsif ($view == $Meta::LIST) {
        return list();	
    }


    
    # These windows do need the meeta data address
    Args::check_meta('meta'); 
    if ($view == $Meta::STATS) {
        return stats();	
    }
    elsif ($view == $Meta::EXPORT) {
        return export();	
    }
    elsif ($view == $Meta::FFIND) {
        return findfile();	
    }
    elsif ($view == $Meta::REPORT) {
        return report();	
    }
    else {
        Print::print_check_err("Invalid Meta View");
    }

}



# Print the two frames
sub frame {

    Print::print_html_header_frameset("Meta Data Browse on $Args::args{'img'}");
    print "<frameset cols=\"20%,80%\">\n";

    # Print the frame where an addres can be entered and a frame for the
    # contents
    if (exists $Args::enc_args{'meta'}) {
        print 
          "<frame src=\"$::PROGNAME?mod=$::MOD_META&view=$Meta::ENTER&$Args::baseargs".
	  "&meta=$Args::enc_args{'meta'}\">\n".
          "<frame src=\"$::PROGNAME?mod=$::MOD_META&view=$Meta::STATS&".
          "meta=$Args::enc_args{'meta'}&$Args::baseargs\" ".
          "name=\"content\">\n</frameset>\n";
    }
    else {
        print 
          "<frame src=\"$::PROGNAME?mod=$::MOD_META&view=$Meta::ENTER&$Args::baseargs\">\n".
          "<frame src=\"$::PROGNAME?mod=$::MOD_META&view=$Meta::BLANK&$Args::baseargs\" ".
          "name=\"content\">\n</frameset>\n";
    }

    Print::print_html_footer_frameset();
    return 0;
};

# Generate the frame to enter the data into
sub enter {
	Print::print_html_header("");

	my $ftype = Args::get_ftype();

	# Address
	print "<form action=\"$::PROGNAME\" method=\"get\" target=\"content\">\n".
	  "<b>$Fs::meta_str{$ftype} Number:</b><br>&nbsp;&nbsp&nbsp;&nbsp;".
	  "<input type=\"text\" name=\"meta\" size=12 maxlength=12";

	print " value=\"$Args::enc_args{'meta'}\"" if exists ($Args::args{'meta'});

	print ">\n".
	  "<input type=\"hidden\" name=\"mod\" value=\"$::MOD_META\">\n".
	  "<input type=\"hidden\" name=\"view\" value=\"$Meta::STATS\">\n".
	  "<input type=\"hidden\" name=\"img\" value=\"$Args::args{'img'}\">\n".
	  Args::make_hidden().

	# Ok Button
	  "<p><input type=\"image\" src=\"pict/but_ok.jpg\" ".
	  "width=43 height=20 alt=\"Ok\" border=\"0\"></form>\n";

	# Allocation List
	print "<p>".
	  "<a href=\"$::PROGNAME?mod=$::MOD_META&view=$Meta::LIST&$Args::baseargs\" target=\"content\">".
	  "<img src=\"pict/but_alloc_list.jpg\" border=\"0\" ".
	  "width=113 height=20 alt=\"Allocation List\">".
	  "</a>\n";

	Print::print_html_footer();
	return 0;
};




# Display the contents of meta
sub stats {
	Print::print_html_header("");

	my $meta = Args::get_meta('meta');
	my $img = Args::get_img('img');
	my $ftype = Args::get_ftype();
	my $tz = "";
	$tz = "-z '$Caseman::tz'" unless ("$Caseman::tz" eq "");

	Print::log_host_inv ("$Args::args{'img'}: Displaying details of $Fs::meta_str{$ftype} $meta");

	my $meta_int = $meta;
	$meta_int = $1 if ($meta =~ /^(\d+)-\d+(-\d)?/);

	my $prev = $meta_int - 1;
	my $next = $meta_int + 1;

	print "<font size=3><center>\n";
	print "<a href=\"$::PROGNAME?mod=$::MOD_META&view=$Meta::STATS&$Args::baseargs&meta=$prev\">".
	  "<img src=\"pict/but_prev.jpg\" alt=\"previous\" ".
	  "width=\"89\" height=20 border=\"0\"></a>\n" 
	  unless ($prev < $Fs::first_meta{$ftype});

	print "<a href=\"$::PROGNAME?mod=$::MOD_META&view=$Meta::STATS&$Args::baseargs&meta=$next\">".
	  "<img src=\"pict/but_next.jpg\" alt=\"next\" ".
	  "width=\"89\" height=20 border=\"0\"></a>\n<br>";

	print "<table cellspacing=\"0\" cellpadding=\"2\">\n<tr>".
	  "<td><a href=\"$::PROGNAME?mod=$::MOD_META&view=$Meta::REPORT&$Args::baseargs&meta=$meta\"".
	  " target=\"_blank\">".
	  "<img src=\"pict/but_report.jpg\" alt=\"report\" ".
	  "width=88 height=20 border=\"0\">".
	  "</a></td>\n".
	  "<td><a href=\"$::PROGNAME?mod=$::MOD_FILE&view=$File::CONT_FR&$Args::baseargs&meta=$meta&".
	  "dir=$Args::args{'img'}-meta-$meta\" target=\"_blank\">".
	  "<img src=\"pict/but_view.jpg\" alt=\"view contents\" ".
	  "width=123 height=20 border=\"0\">".
	  "</a></td>\n".
	  "<td><a href=\"$::PROGNAME?mod=$::MOD_META&view=$Meta::EXPORT&$Args::baseargs&meta=$meta&\">".
	  "<img src=\"pict/but_export.jpg\" alt=\"export\" ".
	  "width=123 height=20 border=\"0\">".
	  "</a></td>";

	print 
	  "<td><a href=\"$::PROGNAME?mod=$::MOD_NOTES&view=$Notes::ENTER_FILE&$Args::baseargs&meta=$meta&\" ".
	  "target=\"_blank\">".
	  "<img src=\"pict/but_addnote.jpg\" alt=\"Add Note\" ".
	  "width=\"89\" height=20 border=\"0\">".
	  "</a></td>" if ($::USE_NOTES == 1);

	print "</tr></table>\n</font></center>\n<font size=3>";

	my $tmpr = Args::get_mnt();
        


	if ($ftype =~ /fat/) {
		print "<a href=\"$::PROGNAME?mod=$::MOD_META&view=$Meta::FFIND&$Args::baseargs&".
		  "meta=$meta\" target=\"_blank\">Find File</a><br>";
	} else {

		print "<b>Pointed to by file:</b><br>\n"; 

		local *OUT;
		Exec::exec_pipe(*OUT, "'$::TSKDIR/ffind' -f $ftype -a '$img' $meta");
		my $cnt = 0;
		while ( $_ = Exec::read_pipe_line(*OUT) ) {
			chop ;
			if (/^(\*)\s+\/*(.*)$/) {
				Print::print_output("<tt><font color=\"$::DEL_COLOR[0]\">$tmpr$2</font></tt> (deleted)<br>\n");
			} elsif (/^\/(.*)$/) {
				Print::print_output("<tt>$tmpr$1</tt><br>\n");
			} else {
				Print::print_output("$_<br>\n");
			}
			$cnt++;
		}
		close (OUT);
		if ($cnt == 0) {
			print "<br>Invalid $Fs::meta_str{$ftype} value<br>\n";
			return;
		}
	}

	Exec::exec_pipe(*OUT, 
	  "'$::TSKDIR/icat' -f $ftype '$img' $meta | '$::TSKDIR/file' -z -b -");
	my $file_type = Exec::read_pipe_line(*OUT);
	close (OUT);

	$file_type = "Error getting file type"
	  if ((!defined $file_type) || ($file_type eq ""));

	print "<b>File Type:</b><br>$file_type<br>\n";

	# MD5 Value
	Exec::exec_pipe(*OUT, 
	  "'$::TSKDIR/icat' -f $ftype '$img' $meta | '$::TSKDIR/md5'");
	my $md5out = Exec::read_pipe_line(*OUT);
	close (OUT);

	$md5out = "Error getting MD5" 
	  if ((!defined $md5out) || ($md5out eq ""));

	chomp $md5out;
	print "<b>MD5:</b><br><tt>$md5out</tt><br>\n";


	# Hash Database Lookups
	if ((($::NSRLDB ne "") || ($Caseman::alert_db ne "") || ($Caseman::exclude_db ne "")) &&
	  ($md5out =~ /^$::REG_MD5$/o)) {

	    print "<form action=\"$::PROGNAME\" method=\"get\" target=\"_blank\">\n".
		  Args::make_hidden().
		  "<input type=\"hidden\" name=\"md5\" value=\"$md5out\">\n".
		  "<input type=\"hidden\" name=\"mod\" value=\"$::MOD_HASH\">\n".
                  "<input type=\"hidden\" name=\"view\" value=\"$Hash::DB_LOOKUP\">\n".
		  "<table cellpadding=\"2\" cellspacing=\"8\"><tr>\n";

		if ($::NSRLDB ne "") {
	  		print "<td align=\"left\">".
			  "<input type=\"checkbox\" name=\"hash_nsrl\" value=\"1\" CHECKED>".
			  "NSRL</td>\n";
		}
		if ($Caseman::alert_db ne "") {
	  		print "<td align=\"left\">".
			  "<input type=\"checkbox\" name=\"hash_alert\" value=\"1\" CHECKED>".
			  "Alert Database</td>\n";
		}
		if ($Caseman::exclude_db ne "") {
	  		print "<td align=\"left\">".
			  "<input type=\"checkbox\" name=\"hash_exclude\" value=\"1\" CHECKED>".
			  "Exclude Database</td>\n";
		}
		print "<td align=\"left\">".
		  "<input type=\"image\" src=\"pict/but_ok.jpg\" ".        
		  "width=43 height=20 alt=\"Ok\" border=\"0\">".
		  "</td></tr></table>\n</form>\n";
	}


	# istat output
	print "<b>Details:</b><br>\n";
	my $mode = 0;	# set to 1 when showing blocks
	my $force = 0;	# set to 1 if size of meta is 0 

	my @output;
	if (exists ($Args::args{'force'})) {
		my $f = Args::get_force();
		Exec::exec_pipe(*OUT,
		  "'$::TSKDIR/istat' -f $ftype $tz -s $Caseman::ts -b $f '$img' $meta");
	} else {
		Exec::exec_pipe(*OUT, 
		  "'$::TSKDIR/istat' -f $ftype $tz -s $Caseman::ts '$img' $meta");
	}
	while ($_ = Exec::read_pipe_line(*OUT) ) {
		if ($mode == 1) {
			if (/^Indirect Blocks/) {
				print "$_<br>\n";
				next;
			}
			elsif (/^Type: (\S+) \((\d+\-\d+)\) (.*)$/) {
				print "Type: $1 (".
				  "<a href=\"$::PROGNAME?mod=$::MOD_FILE&view=$File::CONT_FR&$Args::baseargs".
				  "&meta=$meta_int-$2&dir=$Args::args{'img'}-meta-$meta_int-$2\" ".
				  "target=\"_blank\">$2</a>) $3<br>\n";
				next;
			}

			my $blk;
			foreach $blk (split (/ /, $_)) {
				print "<a href=\"$::PROGNAME?mod=$::MOD_FRAME&submod=$::MOD_DATA&".
				  "$Args::baseargs&block=$blk\" target=\"_top\">$blk</a>  ";
			}
			print "<br>\n";
		}
		else {
			if (/^Not Allocated$/) {
				print "<font color=\"$::DEL_COLOR[0]\">$_</font><br>\n";
			}
			else {
				print "$_<br>\n";
			}
			$mode = 1 if (/^Direct Blocks|^Sectors/);
			$mode = 1 if (/^Attributes/);

			if ((/^size: (\d+)/) && ($1 == 0)) {
				$force = 1;
			}
		}
	}
	close (OUT);

	# display a text box to force X number of blocks to be displayed
	if ($force == 1)  {
	    print "<form action=\"$::PROGNAME\" method=\"get\">\n".
		  Args::make_hidden().
		  "<input type=\"hidden\" name=\"img\" value=\"$Args::args{'img'}\">\n".
		  "<input type=\"hidden\" name=\"meta\" value=\"$meta\">\n".
                  "<input type=\"hidden\" name=\"mod\" value=\"$::MOD_META\">\n".
                  "<input type=\"hidden\" name=\"view\" value=\"$Meta::STATS\">\n";

        print "Enter number of $Fs::addr_unit{$ftype}s to display: <input type=\"text\" ".
		  "value=5 name=\"force\" size=\"3\">\n";

		print "<input type=\"image\" src=\"pict/but_force.jpg\" ".
			  "width=53 height=20 alt=\"Force\" border=\"0\"> (because the size is 0)\n</form>\n";
	}

	Print::print_html_footer();
    return 0;
};


sub findfile {

	Print::print_html_header("Find File");

	my $meta = Args::get_meta('meta');
	my $img = Args::get_img('img');
	my $ftype = Args::get_ftype();

	my $tmpr = Args::get_mnt();
	print "<b>Pointed to by file:</b><br>\n"; 

	local *OUT;
	Exec::exec_pipe(*OUT, "'$::TSKDIR/ffind' -f $ftype -a '$img' $meta");
	while ( $_ = Exec::read_pipe_line(*OUT) ) {
		chop ;
		if (/(\*)\s+\/*(.*)/) {
			Print::print_output("<tt><font color=\"$::DEL_COLOR[0]\">$tmpr$2</font></tt> (deleted)<br>\n");
		} elsif (/^\/(.*)$/) {
			Print::print_output("<tt>$tmpr$1</tt><br>\n");
		} else {
			Print::print_output("$_<br>\n");
		}
	}
	close (OUT);
        
	Print::print_html_footer();
	return 0;
};



sub export {
	my $meta = Args::get_meta('meta');
	my $img = Args::get_img('img');
	my $ftype = Args::get_ftype();

	Print::log_host_inv ("$Args::args{'img'}: Saving contents of $Fs::meta_str{$ftype} $meta");

	Print::print_oct_header(
	  "$Args::args{'img'}"."-meta"."$meta".".raw");

	local *OUT;
	Exec::exec_pipe(*OUT, 
	  "'$::TSKDIR/icat' -f $ftype '$img' $meta");
	print "$_" while ( $_ = Exec::read_pipe_data(*OUT, 512) );
	close (OUT);

	Print::print_oct_footer();
};


sub report {

	my $meta = Args::get_meta('meta');
	my $img = Args::get_img('img');
	my $ftype = Args::get_ftype();
	my $tz = "";
	$tz = "-z '$Caseman::tz'" unless ("$Caseman::tz" eq "");

	Print::log_host_inv ("$Args::args{'img'}: Generating report for $Fs::meta_str{$ftype} $meta");

	Print::print_text_header(
	  "filename=$Args::args{'img'}-meta$meta.txt");

	print "                 Autopsy $Fs::meta_str{$ftype} Report\n\n".
	  "-" x 62 ."\n".
	      "                    GENERAL INFORMATION\n\n".
	  "$Fs::meta_str{$ftype}: $Args::args{'meta'}\n";

	print "Pointed to by file(s):\n"; 
	my $tmpr = Args::get_mnt();
	local *OUT;

	Exec::exec_pipe(*OUT, "'$::TSKDIR/ffind' -f $ftype -a '$img' $meta");
	while ($_ = Exec::read_pipe_line(*OUT) ) {
		chop ;
		if (/^(\*)\s+\/*(.*)$/) {
			Print::print_output("  $tmpr$2 (deleted)\n");
		} elsif (/^\/(.*)$/) {
			Print::print_output("  $tmpr$1\n");
		} else {
			Print::print_output("  $_\n");
		}
	}
	close(OUT);

	Exec::exec_pipe(*OUT, "'$::TSKDIR/istat' -f $ftype $tz -s $Caseman::ts '$img' $meta | '$::TSKDIR/md5'");
	my $md5 = Exec::read_pipe_line(*OUT);
	close (OUT);

	$md5 = "Error getting MD5 Value"
	  if ((!defined $md5) || ($md5 eq ""));

	chop $md5;
	print "MD5 of istat output: $md5\n";

	print "\nImage: $img\n";
	print "Image Type: $ftype\n";

	my $date = localtime();
	print "\nDate Generated: $date\n".
	  "Investigator: $Args::args{'inv'}\n\n".
	  "-" x 62 ."\n".
	  "                   META DATA INFORMATION\n\n";

	Exec::exec_pipe(*OUT,
	  "'$::TSKDIR/istat' -f $ftype $tz -s $Caseman::ts '$img' $meta");
	while ( $_ = Exec::read_pipe_line(*OUT) ) {
       	print "$_";
	}
	close (OUT);

	Exec::exec_pipe (*OUT,
	  "'$::TSKDIR/icat' -f $ftype '$img' $meta | '$::TSKDIR/file' -z -b -");
	my $file_type = Exec::read_pipe_line(*OUT);
	close (OUT);

	$file_type = "Error getting file type"
	  if ((!defined $file_type) || ($file_type eq ""));

	print "\nFile Type: $file_type";

    print "\n"."-" x 62 ."\n".
      "                   VERSION INFORMATION\n\n".
      "Autopsy Version: $::VER\n";
    print "The Sleuth Kit Version: ". ::get_tskver() ."\n";

	Print::print_text_footer();

	return 0;
};


# Display the meta Allocation Table
sub list {
my $ILS_GAP = 500;
 
    my $ftype = Args::get_ftype();
    my $img = Args::get_img('img');

    my $min = 0;

    $min = Args::get_min() if (exists $Args::args{'min'});
    my $max = $min + $ILS_GAP - 1;  

	# Because we can not use metas 0 and 1 for most FS, set fmin to the
	# minimum for this fs
	my $fmin = $min;
	$fmin = $Fs::first_meta{$ftype} if ($min < $Fs::first_meta{$ftype});
        
	Print::print_html_header("$Fs::meta_str{$ftype} Allocation List $fmin -&gt $max");

	Print::log_host_inv ("$Args::args{'img'}: $Fs::meta_str{$ftype} Allocation List for $min to $max");

	print "<center><H2>$Fs::meta_str{$ftype}: $fmin - $max</H2>";
        
	# Display next and previous links
    my $tmp;
    print "<font size=3>\n";
    if ($min > $Fs::first_meta{$ftype}) {
        $tmp = $min - $ILS_GAP;
        print "<a href=\"$::PROGNAME?mod=$::MOD_META&view=$Meta::LIST&$Args::baseargs&min=$tmp\">".
           "<img src=\"pict/but_prev.jpg\" alt=\"previous\" ".
		   "width=\"89\" height=20 border=\"0\"></a> ";
    }
    $tmp = $min + $ILS_GAP;
    print " <a href=\"$::PROGNAME?mod=$::MOD_META&view=$Meta::LIST&$Args::baseargs&min=$tmp\">".
      "<img src=\"pict/but_next.jpg\" alt=\"next\" ".
	  "width=\"89\" height=20 border=\"0\"></a><br>";
    print "</font></center>\n";

	# The list
	local *OUT;
	Exec::exec_pipe (*OUT,
	  "'$::TSKDIR/ils' -e -s $Caseman::ts -f $ftype '$img' $fmin-$max");
	while ( $_ = Exec::read_pipe_line(*OUT) ) {
		if (/^($::REG_META)\|([af])\|\d+\|\d+\|\d+\|\d+\|\d+\|/o) {
			print "<a href=\"$::PROGNAME?mod=$::MOD_META&view=$Meta::STATS&$Args::baseargs&meta=$1\">".
			  "$1:</a> ";
			if ($2 eq "a") {
				print "allocated<br>\n";
			} else {
				print "<font color=\"$::DEL_COLOR[0]\">free</font><br>\n";
			}
		}
	}
	close (OUT);

	# Display next and previous links
    print "<center><font size=3>\n";
    if ($min > $Fs::first_meta{$ftype}) {
        $tmp = $min - $ILS_GAP;
        print "<a href=\"$::PROGNAME?mod=$::MOD_META&view=$Meta::LIST&$Args::baseargs&min=$tmp\">".
           "<img src=\"pict/but_prev.jpg\" alt=\"previous\" ".
		   "width=\"89\" height=20 border=\"0\"></a> ";
    }
    $tmp = $min + $ILS_GAP;
    print " <a href=\"$::PROGNAME?mod=$::MOD_META&view=$Meta::LIST&$Args::baseargs&min=$tmp\">".
      "<img src=\"pict/but_next.jpg\" alt=\"next\" ".
	  "width=\"89\" height=20 border=\"0\"></a><br>";
    print "</font></center>\n";
    
	Print::print_html_footer();
    return 0;

};

# Blank Page
sub blank { 
	Print::print_html_header("");
	print "<!-- This Page Intentionally Left Blank -->\n";
	Print::print_html_footer();
	return 0;
};

